@@ -1201,22 +1201,23 @@ def unique_name(start, contains):
       A name that is not contained in `contains`
     """
     # exit early if name is not in bundle
-    if len(start) > 0 and start not in contains:
+    if len(contains) == 0 or (len(start) > 0 and start not in contains):
         return start
+
     increment = 0
-    if len(start) == 0:
-        formatter = '{}'
-    else:
+    formatter = start + '_{}'
+    if len(start) > 0:
         # split by our delimiter once
         split = start.rsplit('_', 1)
         if len(split) == 2 and split[1].isnumeric():
+            if split[0] not in contains:
+                return split[0]
             # start incrementing from the passed value
             increment = int(split[1])
-        # keep the original name and add an integer to it
-        formatter = split[0] + '_{}'
+            formatter = split[0] + '_{}'
 
     # if contains is empty we will only need to check once
-    for i in range(increment + 1, 1 + increment + len(contains)):
+    for i in range(increment + 1, 2 + increment + len(contains)):
         check = formatter.format(i)
         if check not in contains:
             return check
@@ -1363,6 +1364,7 @@ def _read_buffers(header,
                 names_original[index].append(name)
                 # make name unique across multiple meshes
                 name = unique_name(name, meshes)
+
                 if mode == _GL_LINES:
                     # load GL_LINES into a Path object
                     from ..path.entities import Line
@@ -1435,7 +1437,7 @@ def _read_buffers(header,
                     # each primitive gets it's own Trimesh object
                     if len(m["primitives"]) > 1:
                         kwargs['metadata']['from_gltf_primitive'] = True
-                        name += "_{}".format(j)
+                        name = unique_name(name, meshes)
                     else:
                         kwargs['metadata']['from_gltf_primitive'] = False
 
@@ -1447,6 +1449,8 @@ def _read_buffers(header,
                 else:
                     log.warning('skipping primitive with mode %s!', mode)
                     continue
+                # this should absolutely not be stomping on itself
+                assert name not in meshes
                 meshes[name] = kwargs
                 mesh_prim[index].append(name)
         except BaseException as E:
@@ -1455,6 +1459,7 @@ def _read_buffers(header,
                           exc_info=True),
             else:
                 raise E
+
     # sometimes GLTF "meshes" come with multiple "primitives"
     # by default we return one Trimesh object per "primitive"
     # but if merge_primitives is True we combine the primitives
@@ -1463,18 +1468,21 @@ def _read_buffers(header,
         # if we are only returning one Trimesh object
         # replace `mesh_prim` with updated values
         mesh_prim_replace = dict()
-        mesh_pop = []
+        # these are the names of meshes we need to remove
+        mesh_pop = set()
         for mesh_index, names in mesh_prim.items():
             if len(names) <= 1:
                 mesh_prim_replace[mesh_index] = names
                 continue
-            name = names_original[mesh_index][0]
-            if name in meshes:
-                name = name + '_' + str(np.random.random())[2:12]
+
+            # just take the shortest name option available
+            name = min(names)
             # remove the other meshes after we're done looping
-            mesh_pop.extend(names[:])
-            # collect the meshes
-            # TODO : use mesh concatenation with texture support
+            # since we're reusing the shortest one don't pop
+            # that as we'll be overwriting it with the combined
+            mesh_pop.update(set(names).difference([name]))
+
+            # get all meshes for this group
             current = [meshes[n] for n in names]
             v_seq = [p['vertices'] for p in current]
             f_seq = [p['faces'] for p in current]
@@ -1484,9 +1492,9 @@ def _read_buffers(header,
             for i, p in enumerate(current):
                 face_materials += [i] * len(p['faces'])
             visuals = visual.texture.TextureVisuals(
-                material=visual.material.MultiMaterial(materials=materials),
-                face_materials=face_materials
-            )
+                material=visual.material.MultiMaterial(
+                    materials=materials),
+                face_materials=face_materials)
             if 'metadata' in meshes[names[0]]:
                 metadata = meshes[names[0]]['metadata']
             else:
